import os

from .tools.configuration import ConfigurationTool


class Validator(object):
	@classmethod
	def findCloudformationTemplate(cls):
		""" Search for an Cloudformation Template

			Returns path to template
		"""
		pass

	@classmethod
	def hasPeteSetup(cls):
		""" Check if pete configure has been run
		"""
		# Check if the CI env var exists
		if "PETE_CI" in os.environ:
			# We are going to use the CI environment variables
			return True

		# Check the path
		path = ConfigurationTool._getGlobalPath()

		# Check if the path exists
		if os.path.exists(path) is False:
			return False
		return True

	@classmethod
	def hasPeteProjectSetup(cls):
		""" Check if pete init has been run
		"""
		# Check the path
		path = ConfigurationTool._getProjectPath()

		# Check if the path exists
		if os.path.exists(path) is False:
			return False
		return True



import sys
from termcolor import cprint as print

from .iaction import IAction
from .deployment.cloudformationdeployment import CloudFormationDeployment
from .deployment.amplifydeployment import AmplifyDeployment
from .deployment.hookdeployment import HookDeployment
from .deployment.zappadeployment import ZappaDeployment
from ..enums import EnvironmentType, HookTypes
from ..tools.hooksmanager import hooksManager


class DeploymentAction(IAction):
	""" The deployment action
	"""

	def __init__(self):
		""" Init the action
		"""
		# Initialize the parent
		super().__init__()

		# Set the default environment to development
		self.environment = EnvironmentType.DEVELOPMENT

	def start(self, environType, **kwargs):
		""" Start the deployment
		"""
		# Read all the hooks
		result = hooksManager.readHooks()
		if result is False:
			sys.exit(1)

		# Remember if we found a deployment option
		found = False
		error = False

		# Execute the PRE_FLIGHT hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_FLIGHT)

		# Check if we should execute CloudFormationDeployment
		if CloudFormationDeployment.shouldExecute() is True:
			print("Starting CloudFormation deployment", "blue")
			result = CloudFormationDeployment().start(environType, **kwargs)
			if result is False:
				error = True
			found = True

		# Check if we should execute AmplifyDeployment
		if AmplifyDeployment.shouldExecute() is True:
			print("Starting Amplify deployment", "blue")
			result = AmplifyDeployment().start(environType, **kwargs)
			if result is False:
				error = True
			found = True

		# Check if we should execute ZappaDeployment
		if ZappaDeployment.shouldExecute() is True:
			print("Starting Zappa deployment", "blue")
			result = ZappaDeployment().start(environType, **kwargs)
			if result is False:
				error = True
			found = True

		# Check if we should execute HookDeployment
		if HookDeployment.shouldExecute() is True:
			print("Starting Custom hooks deployment", "blue")
			result = HookDeployment().start(environType, **kwargs)
			if result is False:
				error = True
			found = True

		# Execute the PRE_FLIGHT hooks
		hooksManager.executeHooks(environType, HookTypes.POST_FLIGHT)

		# Check if we found a deployment option
		if found is False:
			print("Could not find any supported deployment methodes. Use Amplify, CloudFormation or Zappa", "red")
			sys.exit(1)

		# Check if an error occured
		if error is True:
			# Set the exit code
			sys.exit(1)



CLOUDFORMATION_TEMPLATE = """
AWSTemplateFormatVersion: '2010-09-09'

Parameters:
  s3FileName:
    Type: String

  environment:
    Type: String

  deploymentBucket:
    Type: String

Resources:
  # Place your AWS resources here
"""



import json
from enum import Enum, auto as autoEnum


from cfn_tools import load_yaml, dump_json


class DevSuffixEnum(Enum):
	ITEM_NAME = autoEnum()


class TemplateTool(object):
	DEV_SUFFIX_TYPES = {
		"AWS::Lambda::Function": DevSuffixEnum.ITEM_NAME,
		"AWS::DynamoDB::Table": "TableName",
		"AWS::AppSync::GraphQLApi": "Name",
		"AWS::IAM::Role": DevSuffixEnum.ITEM_NAME
	}

	NO_TYPE_TAGS_SUPPORT = [
		"AWS::ApiGateway::Deployment",
		"AWS::ApiGateway::Method",
		"AWS::ApiGateway::Resource",
		"AWS::AppSync::DataSource",
		"AWS::AppSync::GraphQLSchema",
		"AWS::AppSync::Resolver",
		"AWS::Events::Rule",
		"AWS::IAM::Role",
		"AWS::Lambda::Permission",
		"AWS::SSM::Parameter",
		"AWS::EC2::LaunchTemplate",
		"AWS::AutoScaling::AutoScalingGroup",
		"AWS::EC2::EIPAssociation",
		"AWS::EC2::SubnetRouteTableAssociation",
		"AWS::EC2::VPCGatewayAttachment",
		"AWS::EC2::Route",
		"AWS::IAM::InstanceProfile",
		"AWS::ApplicationAutoScaling::ScalableTarget",
		"AWS::Cognito::UserPool",
		"AWS::AppSync::ApiKey",
		"AWS::IAM::ServiceLinkedRole",
		"AWS::Lambda::EventSourceMapping"
	]

	@classmethod
	def parseTemplate(cls, templateContent):
		""" Change the YAML templateContent to a plain JSON format
		"""
		# Read the YAML content
		templateContent = load_yaml(templateContent)

		# Change to JSON
		templateContent = dump_json(templateContent)

		# Load the JSON
		templateContent = json.loads(templateContent)

		# Check if it is valid
		if "Resources" not in templateContent or \
			templateContent['Resources'] is None or \
			len(templateContent['Resources']) == 0:
			raise Exception("You dont not have any Resource in your template")

		return templateContent

	@classmethod
	def addSuffixToItems(cls, templateContent):
		""" Add the suffix to the items in the templateContent
		"""
		# Walk through all the items
		for itemName in (list((templateContent['Resources']).keys())):
			# Get the information
			itemType = templateContent['Resources'][itemName]['Type']

			# Check if we now what to change
			if itemType in cls.DEV_SUFFIX_TYPES:
				# Check if we change the name of the item
				if cls.DEV_SUFFIX_TYPES[itemType] == DevSuffixEnum.ITEM_NAME:
					newItemName = itemName + "Dev"
					templateContent['Resources'][newItemName] = templateContent['Resources'][itemName]
					del templateContent['Resources'][itemName]

				# Change the mentioned thing
				else:
					# Check if properties exists
					if "Properties" not in templateContent['Resources'][itemName]:
						templateContent['Resources'][itemName]['Properties'] = {}

					# Change the name
					templateContent['Resources'][itemName]['Properties'][cls.DEV_SUFFIX_TYPES[itemType]] = templateContent['Resources'][itemName]['Properties'][cls.DEV_SUFFIX_TYPES[itemType]] + "_DEV"

			# Just try it
			else:
				# Check if properties exists
				if "Properties" not in templateContent['Resources'][itemName]:
						templateContent['Resources'][itemName]['Properties'] = {}

				# Check if name exists
				if "Name" in templateContent['Resources'][itemName]['Properties']:
					templateContent['Resources'][itemName]['Properties']['Name'] = templateContent['Resources'][itemName]['Properties']['Name'] + "_DEV"

				# Check if ID exists
				elif "Id" in templateContent['Resources'][itemName]['Properties']:
					templateContent['Resources'][itemName]['Properties']['Id'] = templateContent['Resources'][itemName]['Properties']['Id'] + "_DEV"
				else:
					raise Exception("I dont know how to change the name of this item '%s'" % itemName)

		return templateContent

	@classmethod
	def addTagsToItems(cls, templateContent):
		""" Add tags to each resource in the templateContent
		"""
		# Walk through all the items
		for itemName in (list((templateContent['Resources']).keys())):
			# Get the information
			itemType = templateContent['Resources'][itemName]['Type']

			# Check if we should add the tags
			if itemType in cls.NO_TYPE_TAGS_SUPPORT:
				continue

			# Check if properties exists
			if "Properties" not in templateContent['Resources'][itemName]:
				templateContent['Resources'][itemName]['Properties'] = {}

			# Check if tags already exists
			if "Tags" not in templateContent['Resources'][itemName]['Properties']:
				templateContent['Resources'][itemName]['Properties']['Tags'] = []

			# Get the key names
			keyNames = [tag['Key'] for tag in templateContent['Resources'][itemName]['Properties']['Tags']]

			# Add the name tag
			if "Name" not in keyNames:
				(templateContent['Resources'][itemName]['Properties']['Tags']).append({"Key": "Name", "Value": itemName})

			# Add the environment tag
			if "Environment" not in keyNames:
				(templateContent['Resources'][itemName]['Properties']['Tags']).append({"Key": "Environment", "Value": {"Ref": "environment"}})

			# Add the stack tag
			if "Stack" not in keyNames:
				(templateContent['Resources'][itemName]['Properties']['Tags']).append({"Key": "Stack", "Value": {"Ref": "stackName"}})

			# Add the project tag
			if "Project" not in keyNames:
				(templateContent['Resources'][itemName]['Properties']['Tags']).append({"Key": "Project", "Value": {"Ref": "projectName"}})

		return templateContent

	@classmethod
	def checkVariables(cls, templateContent):
		""" Check if all the required variables are added
		"""
		# Remember all the parameters
		parameters = {}

		# Check if there are parameters
		if "Parameters" not in templateContent:
			templateContent['Parameters'] = {}

		# Check if all exists
		if "s3FileName" not in templateContent['Parameters']:
			templateContent['Parameters']['s3FileName'] = {"Type": "String"}
		if "environment" not in templateContent['Parameters']:
			templateContent['Parameters']['environment'] = {"Type": "String"}
		if "deploymentBucket" not in templateContent['Parameters']:
			templateContent['Parameters']['deploymentBucket'] = {"Type": "String"}
		if "stackName" not in templateContent['Parameters']:
			templateContent['Parameters']['stackName'] = {"Type": "String"}
		if "projectName" not in templateContent['Parameters']:
			templateContent['Parameters']['projectName'] = {"Type": "String"}

		# Check for other parameters
		for parameterName in templateContent['Parameters']:
			# Check if this is an Pete parameter
			if parameterName in ["s3FileName", "environment", "deploymentBucket", "stackName", "projectName"]:
				continue

			# Get the type of parameter
			parameterType = "String"
			if "Type" in templateContent['Parameters'][parameterName]:
				parameterType = templateContent['Parameters'][parameterName]['Type']

			# Save the parameter
			parameters[parameterName] = parameterType

		return (templateContent, parameters)



from enum import Enum


class EnvironmentType(Enum):
	""" Different Environment types
	"""
	DEVELOPMENT = 1
	PRODUCTION = 2


class HookTypes(Enum):
	""" Enum with all the different moment when a hook can be executed
	"""
	# First step of the deployment
	PRE_FLIGHT = 1
	POST_FLIGHT = 2

	# Before things are being uploaded; Example: CloudFormation S3 Upload
	PRE_UPLOAD = 3
	POST_UPLOAD = 4

	# Before files are being compiles; Example: CloudFormation template check
	PRE_COMPILE = 5
	POST_COMPILE = 6

	# Before things are being executed; Example: CloudFormation is being executed
	PRE_DEPLOYMENT = 7
	POST_DEPLOYMENT = 8

	# Before things are installed; Example: Dependency install
	PRE_INSTALL = 9
	POST_INSTALL = 10



from .enums import EnvironmentType, HookTypes


class IHook(object):
	""" Basis for all Hooks
	"""
	# Name of the hook
	NAME: str = None

	# What moment should the hook be executed at
	# This is a HookTypes
	ON_TYPE: HookTypes = None

	# What environment should the hook be execute at
	# This is a list of EnvironmentType
	#
	ON_ENVIRONMENT: list = None

	def execute(self, environmentType: EnvironmentType):
		""" Execute the hook
		"""
		raise NotImplementedError


class DeploymentHook(IHook):
	""" A special hook for deployments
	"""



import json
import os
from enum import Enum
from pathlib import Path


class ConfigType(Enum):
	GLOBAL = "global"
	PROJECT = "project"
	LOCAL = "local"


class ConfigKey(Enum):
	DEV_PROFILE = "dev-profile"
	PROD_PROFILE = "prod-profile"
	DEV_BUCKET = "dev-bucket"
	PROD_BUCKET = "prod-bucket"
	DEV_REGION = "dev-region"
	PROD_REGION = "prod-region"
	DEV_SUFFIX = "dev-suffix"
	STACK_NAME = "stack-name"
	PARAMETERS = "parameters"


class ConfigurationTool(object):
	# Place to hold the config
	config = {}

	@classmethod
	def readConfig(cls, getGlobal=True, getProject=True, getLocal=True):
		""" Read the config
		"""
		# Get the configs
		configTypes = []
		if getGlobal is True:
			configTypes.append(ConfigType.GLOBAL)
		if getProject is True:
			configTypes.append(ConfigType.PROJECT)
		if getLocal is True:
			configTypes.append(ConfigType.LOCAL)
		configs = cls._getConfigPaths(configTypes)

		# Empty the config
		cls.config = {}

		# Walk through the paths
		for config in configs:
			# Get the information
			path = config['path']
			configType = config['type']

			# Check if the location exists
			if path is None or os.path.exists(path) is False:
				continue

			# Read the config
			try:
				f = open(path, "r")
				data = json.loads(f.read())
				f.close()
			except Exception:
				raise Exception("Could not read config at '%s'. Delete the file or fix it!" % path)

			# Transform the data
			try:
				for key in data.keys():
					keyName = ConfigKey(key)
					cls.setConfig(keyName, data[key], configType)
			except Exception:
				raise Exception("Could not read config at '%s'. Delete the file or fix it!" % path)

		# Also read environment variables
		cls.readEnvironmentConfig()

	@classmethod
	def readEnvironmentConfig(cls):
		""" Get the config variables from the Config
		"""
		# Check if we are using CI
		if "PETE_CI" in os.environ:
			# Get the data
			keys = ["PETE_DEV_PROFILE", "PETE_PROD_PROFILE", "PETE_DEV_BUCKET", "PETE_PROD_BUCKET", "PETE_DEV_REGION", "PETE_PROD_REGION"]

			# Walk throught the keys
			for key in keys:
				# Check if it exists
				if key in os.environ:
					# Transform the key to ConfigKey
					configKey = key[5:]
					configKey = configKey.lower()
					configKey = configKey.replace("_", "-")
					configKey = ConfigKey(configKey)

					# Save the data
					cls.setConfig(configKey, os.environ[key], ConfigType.GLOBAL)

	@classmethod
	def saveConfig(cls, configType):
		""" Write the config to a file
		"""
		# Get the configs
		configs = cls._getConfigPaths([configType])

		# Walk through the paths
		for config in configs:
			# Get the information
			path = config['path']
			configType = config['type']

			# Check if path exists
			if os.path.exists(os.path.dirname(path)) is False:
				# Create the directories
				os.makedirs(os.path.dirname(path))

			# Replace the ConfigurationKeys
			configData = {}
			for key in cls.config.keys():
				if configType in cls.config[key]:
					if cls.config[key][configType] is not None:
						keyName = key.value
						configData[keyName] = cls.config[key][configType]

			# Write the config
			f = open(path, "w")
			f.write(json.dumps(configData))
			f.close()

	@classmethod
	def getAllConfig(cls, getGlobal=True, getProject=True, getLocal=True, emptyValue=None):
		""" Get all config
		"""
		# Create a temporary config storage
		config = {}

		# Walk throught the config
		for key in cls.config:
			# Check if local has been filled in
			if ConfigType.LOCAL in cls.config[key] and getLocal is True:
				config[key] = cls.config[key][ConfigType.LOCAL]

			# Check if project has been filled in
			if ConfigType.PROJECT in cls.config[key] and getProject is True:
				config[key] = cls.config[key][ConfigType.PROJECT]

			# Check if global has been filled in
			if ConfigType.GLOBAL in cls.config[key] and getGlobal is True:
				config[key] = cls.config[key][ConfigType.GLOBAL]

		# Check if all keys exists
		for key in ConfigKey:
			if key not in config:
				config[key] = emptyValue

		return config

	@classmethod
	def getConfigWithType(cls, key, getGlobal=True, getProject=True, getLocal=True, emptyValue=None):
		""" Get a specific config value with type

			Returns a specific value
		"""
		# Check if type exists
		if key in cls.config:
			# Check if local has been filled in
			if ConfigType.LOCAL in cls.config[key] and getLocal is True:
				return {"value": cls.config[key][ConfigType.LOCAL], "type": ConfigType.LOCAL}

			# Check if project has been filled in
			if ConfigType.PROJECT in cls.config[key] and getProject is True:
				return {"value": cls.config[key][ConfigType.PROJECT], "type": ConfigType.PROJECT}

			# Check if global has been filled in
			if ConfigType.GLOBAL in cls.config[key] and getGlobal is True:
				return {"value": cls.config[key][ConfigType.GLOBAL], "type": ConfigType.GLOBAL}

		return emptyValue

	@classmethod
	def getConfig(cls, key, getGlobal=True, getProject=True, getLocal=True, emptyValue=None):
		""" Get a specific config value

			Returns a specific value
		"""
		# Check if type exists
		if key in cls.config:
			# Check if local has been filled in
			if ConfigType.LOCAL in cls.config[key] and getLocal is True:
				return cls.config[key][ConfigType.LOCAL]

			# Check if project has been filled in
			if ConfigType.PROJECT in cls.config[key] and getProject is True:
				return cls.config[key][ConfigType.PROJECT]

			# Check if global has been filled in
			if ConfigType.GLOBAL in cls.config[key] and getGlobal is True:
				return cls.config[key][ConfigType.GLOBAL]

		return emptyValue

	@classmethod
	def getConfigRow(cls, key, emptyValue=None):
		""" Get a specific config value row

			Returns a specific value
		"""
		# Check if type exists
		if key in cls.config:
				return cls.config[key]

		return emptyValue

	@classmethod
	def setConfig(cls, key, value, configType):
		""" Set a specific config key
		"""
		# Check if the key exists in config
		if key not in cls.config:
			cls.config[key] = {}

		# Save the value
		cls.config[key][configType] = value

	@classmethod
	def _getGlobalPath(cls):
		""" Get the path to the configuration file
		"""
		# Get path to user home dir
		home = str(Path.home())

		# Create the path
		return os.path.join(home, ".pete", "configuration")

	@classmethod
	def _getProjectPath(cls):
		""" Get the path to the project configuration files
		"""
		# Get the current working dir
		workingDir = os.getcwd()
		path = None

		# Search for the pete folder
		while True:
			# Check if we reached the top
			if os.path.ismount(workingDir) is True:
				break

			# Check if a directory exists
			if ".pete" in os.listdir(workingDir):
				path = workingDir
				break

			# Move a directory up
			workingDir = os.path.dirname(workingDir)[0]

		# Check if we found a path
		if path is None:
			path = os.getcwd()

		return os.path.join(path, ".pete", "configuration")

	@classmethod
	def _getLocalPath(cls):
		""" Get the path to the local project configuration files
		"""
		# Get the current working dir
		workingDir = os.getcwd()
		path = None

		# Search for the pete folder
		while True:
			# Check if we reached the top
			if os.path.ismount(workingDir) is True:
				break

			# Check if a directory exists
			if ".pete" in os.listdir(workingDir):
				path = workingDir
				break

			# Move a directory up
			workingDir = os.path.dirname(workingDir)[0]

		# Check if we found a path
		if path is None:
			path = os.getcwd()

		return os.path.join(path, ".pete", "local")

	@classmethod
	def _getConfigPaths(cls, configTypes):
		""" Get the paths for the config files

			Returns a list
		"""
		# Remember the configs we are going to read
		configs = []

		# Check if we need global config
		if ConfigType.GLOBAL in configTypes:
			# Get the file path
			configs.append({"type": ConfigType.GLOBAL, "path": cls._getGlobalPath()})

		# Check if we need project config
		if ConfigType.PROJECT in configTypes:
			# Get the file path
			configs.append({"type": ConfigType.PROJECT, "path": cls._getProjectPath()})

		# Check if we need local config
		if ConfigType.LOCAL in configTypes:
			# Get the file path
			configs.append({"type": ConfigType.LOCAL, "path": cls._getLocalPath()})

		return configs



import os
import subprocess

from termcolor import cprint as print

from ...tools.hooksmanager import hooksManager
from ...tools.configuration import ConfigurationTool
from ...tools.spinner import Spinner
from .ideploymentaction import IDeploymentAction
from ...enums import EnvironmentType, HookTypes


class ZappaDeployment(IDeploymentAction):
	""" Deployment through Zappa
	"""
	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		# Check if there is an zappa folder
		return os.path.exists("zappa_settings.json")

	def start(self, environType, **kwargs):
		""" Start the deployment
		"""
		# Get the configs
		ConfigurationTool.readConfig()

		# Check if there is an template
		if os.path.exists("zappa_settings.json") is False:
			raise Exception("Cant find Zappa template: 'zappa_settings.json'")

		# Remember the environment type for later
		self.environment = environType

		# Check if it is the production environment
		if self.environment == EnvironmentType.PRODUCTION:
			stageName = "production"
		else:
			stageName = "dev"

		# Deploy the stage
		self._deployStage(stageName)

	def _deployStage(self, stageName):
		""" Run the zappa command
		"""
		# Execute the PRE_COMPILE hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_COMPILE)

		# Create a temporary directory
		with Spinner(text="Creating environment") as spinner:
			self._createTempDir()
			spinner.succeed()

		# Execute the POST_COMPILE hooks
		hooksManager.executeHooks(self.environment, HookTypes.POST_COMPILE)

		# Execute the PRE_INSTALL hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_INSTALL)

		# Check the dependencies
		with Spinner(text="Installing dependencies") as spinner:
			self._checkDependencies()
			spinner.succeed()

		# Execute the POST_INSTALL hooks
		hooksManager.executeHooks(self.environment, HookTypes.POST_INSTALL)

		# Execute the PRE_DEPLOYMENT hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_DEPLOYMENT)

		# Zapping it
		with Spinner(text="Zappa-ing") as spinner:
			# Find how to run zappa
			zappaCommand = self._findZappa()

			# Check if we could find a zappa
			if zappaCommand is None:
				raise Exception("Could not find Zappa. Make sure it is installed.")

			# Run the deployment command
			try:
				subprocess.check_output("cd %s && %s deploy %s" % (self.location, zappaCommand, stageName), stderr=subprocess.STDOUT, shell=True)
			except subprocess.CalledProcessError as e:
				# Check if it was already deployed
				if b"did you mean to call update" in e.output:
					# Run the update command
					try:
						subprocess.check_output("cd %s && %s update %s" % (self.location, zappaCommand, stageName), stderr=subprocess.STDOUT, shell=True)
					except subprocess.CalledProcessError as e:
						print(e.output, "red")
						return False
				else:
					print("An error occured", "red")
					return False
			finally:
				# Execute the POST_DEPLOYMENT hooks
				hooksManager.executeHooks(self.environment, HookTypes.POST_DEPLOYMENT)

	def _findZappa(self):
		""" Search for Zappa
		"""
		# Check if zappa is available as a root level
		try:
			subprocess.check_output("zappa", shell=True)
		except subprocess.CalledProcessError:
			pass
		else:
			return "zappa"

		# Check if poetry is available
		if os.path.exists("poetry.lock") is True:
			# Try zappa in poetry
			try:
				subprocess.check_output("poetry run zappa", shell=True)
			except subprocess.CalledProcessError:
				pass
			else:
				return "poetry run zappa"

		return None



import os
import importlib.util

from termcolor import cprint as print

from .ideploymentaction import IDeploymentAction
from ...tools.configuration import ConfigurationTool
from ...ihook import DeploymentHook, IHook


class HookDeployment(IDeploymentAction):
	""" Deployment through Custom hooks
	"""
	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		# Check if there are any custom hooks
		return os.path.exists(".pete/hooks")

	def start(self, environType, **kwargs):
		""" Start the deployment
		"""
		# Get the configs
		ConfigurationTool.readConfig()

		# Save the environment type
		self.environment = environType

		# Check if there is an hooks directory
		if os.path.exists(".pete/hooks") is False:
			return

		# Read the hook files
		hooks = os.listdir(".pete/hooks")

		# Walk throught the hooks
		for fileName in hooks:
			# Check if it is an py file
			if os.path.splitext(fileName)[1] != ".py":
				continue

			print("Running hook %s" % fileName, "blue")

			# Import the file
			spec = importlib.util.spec_from_file_location(os.path.splitext(fileName)[0], os.path.join(".pete", "hooks", fileName))
			module = importlib.util.module_from_spec(spec)
			spec.loader.exec_module(module)

			# Create a hook
			try:
				hook = module.Hook()
			except AttributeError:
				print("There is no Hook class in %s" % fileName, "red")
				return False

			# Check if it is an DeploymentHook
			if isinstance(hook, DeploymentHook) is False:
				if isinstance(hook, IHook) is True:
					continue

				print("Hook %s is not a DeploymentHook subclass" % fileName, "red")
				return False

			# Run the execute
			hook.execute(self.environment)



import os
import subprocess

from termcolor import cprint as print
from .ideploymentaction import IDeploymentAction
from ...tools.hooksmanager import hooksManager
from ...enums import EnvironmentType, HookTypes
from ...tools.spinner import Spinner


class AmplifyDeployment(IDeploymentAction):
	""" Deployment through AWS Amplify
	"""
	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		# Check if there is an amplify folder
		return os.path.exists("amplify")

	def start(self, environType, **kwargs):
		""" Execute AWS Amplify deployment
		"""
		# Check if it is the production environment
		if environType != EnvironmentType.PRODUCTION:
			raise Exception("AWS Amplify can only be deployed in production mode")

		# Execute the PRE_INSTALL hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_INSTALL)

		# Check the dependencies
		with Spinner(text="Installing dependencies") as spinner:
			self._checkDependencies()
			spinner.succeed()

		# Execute the POST_INSTALL hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_INSTALL)

		# Execute the PRE_DEPLOYMENT hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_DEPLOYMENT)

		# Send it to AWS Amplify
		with Spinner(text="AWS Amplify deploying") as spinner:
			status = self._amplifyDeploy()

			# Execute the POST_DEPLOYMENT hooks
			hooksManager.executeHooks(environType, HookTypes.POST_DEPLOYMENT)

			if status is True:
				spinner.succeed()
			else:
				spinner.fail()
			return status

	def _amplifyDeploy(self):
		""" Deploy with AWS Amplify
		"""
		# List with command to run
		listCommands = ["amplify push", "amplify publish"]

		# Execute the command
		for amplifyCommand in listCommands:
			# Create the command
			command = "cd %s && " % self.location
			command = command + amplifyCommand

			print(command)

			# Run the command
			try:
				subprocess.check_call(command, shell=True)
			except Exception:
				return False
		return True



import sys

from PyInquirer import prompt
from termcolor import cprint as print

from ..tools.awscli import AWSCliTool


class IAction(object):
	def start(self, **kwargs):
		pass

	def _askAWSProfile(self, default=None):
		""" Ask for an AWS Profile
		"""
		# Get all the profile from AWS Cli
		profiles = AWSCliTool.getProfiles()

		# Check if there are profile
		if len(profiles) == 0:
			raise Exception("To use Pete, first setup AWS Cli by using: `aws configure`")

		# Ask the question
		answer = prompt({
			"type": "list",
			"name": "profile",
			"choices": profiles,
			"message": "Which profile do you want to use?",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['profile']

	def _askS3Bucket(self, profile, default=None):
		""" Ask for an S3 Bucket
		"""
		# Get all the profile from AWS Cli
		buckets = AWSCliTool.getS3Buckets(profile=profile)

		# Check if there are buckets
		if len(buckets) == 0:
			print("Could not get your S3 bucket, either because you have none or because you dont have permission.", "red")
			print("Enter the name of the S3 bucket you would like to use.", "yellow")
			return self._askName()

		# Ask the question
		answer = prompt({
			"type": "list",
			"name": "bucket",
			"choices": buckets,
			"message": "Which bucket do you want to use?",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['bucket']

	def _askName(self, default=None):
		""" Ask for a name
		"""
		# Ask the question
		answer = prompt({
			"type": "input",
			"name": "name",
			"message": "What name do you want to use?",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['name']

	def _askAWSRegion(self, profile, default=None):
		""" Ask to override AWS region
		"""
		# Get the default region
		defaultRegion = AWSCliTool.getRegion(profile)

		# Ask to override the region
		answer = prompt({
			"type": "confirm",
			"name": "override",
			"message": "Do you want to override the default region (%s)?" % defaultRegion,
			"default": True if default is not None else False
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		# Check the answer
		if answer['override'] is False:
			return None

		# Ask for the region
		answer = prompt({
			"type": "input",
			"name": "region",
			"message": "What region do you want to use?",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['region']

	def _askOverride(self, default=False):
		""" Ask if you want to override
		"""
		# Ask the question
		answer = prompt({
			"type": "confirm",
			"message": "Do you want to override?",
			"name": "override",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['override']

	def _askConfirm(self, message, default=False):
		""" Ask if you want to confirm
		"""
		# Ask the question
		answer = prompt({
			"type": "confirm",
			"message": message,
			"name": "confirm",
			"default": default if default is not None else ""
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['confirm']



import os
from halo import Halo
from termcolor import cprint as print


class Spinner(object):
	""" Show a Spinner.
		But hide it in CI mode
	"""

	def __init__(self, *args, **kwargs):
		""" Initialize
		"""
		# Save the data
		self.args = args
		self.kwargs = kwargs

	def __enter__(self):
		""" Open the spinner
		"""
		# Check if we are in CI mode
		if "PETE_CI" in os.environ:
			# Make a fake spinner
			self.spinner = DummySpinner(*self.args, **self.kwargs)
		else:
			# Make a real spinner
			self.spinner = Halo(*self.args, **self.kwargs)

		# Start the spinner
		self.spinner.start()

		return self.spinner

	def __exit__(self, *args, **kwargs):
		""" Close the spinner
		"""
		self.spinner.stop()


class DummySpinner(object):
	""" This does nothing
	"""
	def __init__(self, *args, **kwargs):
		self.text = ""
		if "text" in kwargs:
			self.text = kwargs["text"]
		print("(EXECUTING) %s" % self.text, "yellow")

	def start(self, *args, **kwargs):
		return

	def stop(self, *args, **kwargs):
		return

	def succeed(self, *args, **kwargs):
		print("(SUCCESS) %s" % self.text, "yellow")

	def fail(self, *args, **kwargs):
		print("(FAILURE) %s" % self.text, "yellow")



import json
import os
import sys
import time
import zipfile

from PyInquirer import prompt
from termcolor import cprint as print

from ...enums import EnvironmentType, HookTypes
from ...tools.hooksmanager import hooksManager
from ...tools.configuration import ConfigurationTool, ConfigKey, ConfigType
from ...tools.template import TemplateTool
from ...tools.boto import BotoTool
from ...tools.spinner import Spinner
from .ideploymentaction import IDeploymentAction


class CloudFormationDeployment(IDeploymentAction):
	""" Deployment through AWS CloudFormation
	"""

	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		# Check if there is an template
		return os.path.exists("template.yaml")

	def start(self, environType, **kwargs):
		""" Start the deployment
		"""
		# Get the configs
		ConfigurationTool.readConfig()

		# Check if there is an template
		if os.path.exists("template.yaml") is False:
			raise Exception("Cant find CloudFormation template: 'template.yaml'")

		# Save the environment type
		self.environment = environType

		# Execute the PRE_COMPILE hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_COMPILE)

		# Create a temporary directory
		with Spinner(text="Creating environment") as spinner:
			self._createTempDir()
			spinner.succeed()

		# Create a new temporary template
		with Spinner(text="Creating template") as spinner:
			try:
				parameters = self._createTemporaryTemplate()
			except Exception as e:
				spinner.fail()
				print(e, "red")
				return False
			else:
				spinner.succeed()

		# Check if there are other parameters
		parameters = self._checkParameters(parameters)

		# Execute the POST_COMPILE hooks
		hooksManager.executeHooks(environType, HookTypes.POST_COMPILE)

		# Execute the PRE_INSTALL hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_INSTALL)

		# Check the dependencies
		with Spinner(text="Installing dependencies") as spinner:
			self._checkDependencies()
			spinner.succeed()

		# Execute the POST_INSTALL hooks
		hooksManager.executeHooks(environType, HookTypes.POST_INSTALL)

		# Execute the PRE_UPLOAD hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_UPLOAD)

		# Zip it all
		with Spinner(text="Zipping content") as spinner:
			zipName = self._zipContent()
			spinner.succeed()

		# Upload the zip
		with Spinner(text="Uploading to S3") as spinner:
			s3Location = self._uploadToS3(zipName)
			spinner.succeed()

		# Execute the POST_UPLOAD hooks
		hooksManager.executeHooks(environType, HookTypes.POST_UPLOAD)

		# Execute the PRE_DEPLOYMENT hooks
		hooksManager.executeHooks(environType, HookTypes.PRE_DEPLOYMENT)

		# Send it to CloudFormation
		with Spinner(text="CloudFormation deploying") as spinner:
			# Execute CloudFormation
			status = self._cloudformationDeploy(parameters, s3Location)

			# Execute the PRE_DEPLOYMENT hooks
			hooksManager.executeHooks(environType, HookTypes.POST_DEPLOYMENT)

			# Check if all is well
			if status is True:
				spinner.succeed()
			else:
				spinner.fail()
				return False
		return True

	def _createTemporaryTemplate(self):
		""" Create a temporary template

			Returns a dict with the parameters
		"""
		# Get the template
		f = open("template.yaml", "r")
		template = f.read()
		f.close()
		template = TemplateTool.parseTemplate(template)

		# Check the environment
		if self.environment == EnvironmentType.DEVELOPMENT:
			if ConfigurationTool.getConfig(ConfigKey.DEV_SUFFIX) is True:
				# Add the dev suffix to the items
				template = TemplateTool.addSuffixToItems(template)

		# Add the tags
		template = TemplateTool.addTagsToItems(template)

		# Check the parameters
		template, parameters = TemplateTool.checkVariables(template)

		# Save the template
		f = open(os.path.join(self.location, ".deployment.template.json"), "w")
		f.write(json.dumps(template))
		f.close()

		# Add the basic parameters
		parameters['environment'] = self.environment
		parameters['stackName'] = ConfigurationTool.getConfig(ConfigKey.STACK_NAME) + ("_development" if ConfigurationTool.getConfig(ConfigKey.DEV_SUFFIX) is True and self.environment == EnvironmentType.DEVELOPMENT else "")
		parameters['projectName'] = ConfigurationTool.getConfig(ConfigKey.STACK_NAME)

		return parameters

	def _zipContent(self):
		""" Zip the current directory

			Returns the zip filename
		"""
		# Create a list of ignored folders
		ignoredFolders = [".git", ".svn", ".pete", ".vscode", "seeders"]

		# Create a filename
		zipFileName = "pete_%s.zip" % int(time.time())

		# Create the zip file
		with zipfile.ZipFile(os.path.join(self.location, zipFileName), "w", zipfile.ZIP_DEFLATED) as zipFile:
			# Walk through all the files
			for root, dirs, files in os.walk(self.location):
				# Check the ignored folders
				goToNext = False
				for ignoredFolderName in ignoredFolders:
					if ignoredFolderName in root:
						goToNext = True
						break

				# Check if we found a ignored folder
				if goToNext is True:
					continue

				# Walk throught the files of the directory
				for fileName in files:
					# Check if this is an zip
					if ".zip" in fileName:
						continue

					# Add them to the zip file
					arcname = os.path.join(root, fileName)[len(self.location):]
					zipFile.write(os.path.join(root, fileName), arcname=arcname)

		return zipFileName

	def _uploadToS3(self, zipName):
		""" Upload the zip file to S3
		"""
		# Create a full filename
		fullFileName = "%s/%s" % ((ConfigurationTool.getConfig(ConfigKey.STACK_NAME).lower()), zipName)

		# Get the bucket name and profile
		bucketName = self._getDeploymentBucketName()
		profileName = self._getDeploymentProfile()
		region = self._getDeploymentRegion()

		# Upload the file
		BotoTool.uploadToS3(
			fromPath=os.path.join(self.location, zipName),
			toBucket=bucketName,
			toKey=fullFileName,
			region=region,
			profile=profileName
		)

		return fullFileName

	def _getDeploymentBucketName(self):
		""" Get the deployment Bucket name from the config

			Returns name of the bucket
		"""
		# Check if we use the development environment
		if self.environment == EnvironmentType.DEVELOPMENT:
			# Check if the DEV_BUCKET is in the projectConfig
			bucketName = ConfigurationTool.getConfig(ConfigKey.DEV_BUCKET)

		# This is the production environment
		else:
			# Check if the PROD_BUCKET is in the projectConfig
			bucketName = ConfigurationTool.getConfig(ConfigKey.PROD_BUCKET)

		return bucketName.strip()

	def _getDeploymentProfile(self):
		""" Get the deployment AWS Profile

			Returns name of profile
		"""
		# Check if we use the development environment
		if self.environment == EnvironmentType.DEVELOPMENT:
			# Check if the DEV_PROFILE is in the projectConfig
			profileName = ConfigurationTool.getConfig(ConfigKey.DEV_PROFILE)

		# This is the production environment
		else:
			# Check if the PROD_PROFILE is in the projectConfig
			profileName = ConfigurationTool.getConfig(ConfigKey.PROD_PROFILE)

		return profileName.strip()

	def _getDeploymentRegion(self):
		""" Get the deployment AWS Region

			Returns name of region
		"""
		# Default option for region
		region = None

		# Check if we use the development environment
		if self.environment == EnvironmentType.DEVELOPMENT:
			# Check if the DEV_REGION is in the projectConfig
			region = ConfigurationTool.getConfig(ConfigKey.DEV_REGION)

		# This is the production environment
		else:
			# Check if the PROD_REGION is in the projectConfig
			region = ConfigurationTool.getConfig(ConfigKey.PROD_REGION)

		return region

	def _cloudformationDeploy(self, parameters, s3Location):
		""" Deploy to CloudFormation
		"""
		# Get the information
		deploymentBucket = self._getDeploymentBucketName()
		profileName = self._getDeploymentProfile()
		region = self._getDeploymentRegion()
		stackName = ConfigurationTool.getConfig(ConfigKey.STACK_NAME)

		# Get the boto client
		client = BotoTool._getClient("cloudformation", region=region, profile=profileName)

		# Check if the stack exists
		stackExists = False
		try:
			client.describe_stacks(StackName=stackName)
		except Exception:
			# There are currently no stacks in CloudFormation
			pass
		else:
			# The stack exists
			stackExists = True

		# Upload the file to S3
		templatePath = os.path.join(self.location, ".deployment.template.json")
		templateName = "%s/template-%s.json" % (stackName.lower(), str(int(time.time())))
		templateUrl = BotoTool.uploadToS3(
			fromPath=templatePath,
			toBucket=deploymentBucket,
			toKey=templateName,
			region=region,
			profile=profileName
		)

		# Create a change set name
		changeStackName = "%s%s" % (stackName, str(int(time.time())))

		# Create the change set parameters
		changeStackParameters = [
			{"ParameterKey": "deploymentBucket", "ParameterValue": deploymentBucket},
			{"ParameterKey": "s3FileName", "ParameterValue": s3Location}
		]

		# Add extra parameters
		for key, value in parameters.items():
			changeStackParameters.append({"ParameterKey": key, "ParameterValue": str(value)})

		# Check if the stack exists
		if stackExists is False:
			# Create the stack
			client.create_stack(
				StackName=stackName,
				TemplateURL=templateUrl,
				Parameters=changeStackParameters,
				Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
				Tags=[
					{"Key": "Stack", "Value": stackName},
				]
			)

		else:
			# Create a change set
			client.create_change_set(
				StackName=stackName,
				ChangeSetName=changeStackName,
				TemplateURL=templateUrl,
				Parameters=changeStackParameters,
				Capabilities=["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
				Tags=[
					{"Key": "Stack", "Value": stackName},
				]
			)

			# Wait for the results
			while True:
				# Get the change set
				changeSet = client.describe_change_set(StackName=stackName, ChangeSetName=changeStackName)

				# Try to get the status
				try:
					stackStatus = changeSet["Status"]
					if "StatusReason" in changeSet:
						statusText = changeSet["StatusReason"]
					else:
						statusText = ""
				except Exception:
					stackStatus = "CREATE_IN_PROGRESS"

				# Check the status
				if stackStatus[-7:] == "_FAILED" or stackStatus == "FAILED":
					print(statusText, "red")
					return False
				elif stackStatus[-9:] == "_COMPLETE":
					break

				# Wait a little
				time.sleep(15)

			# Apply the change set
			client.execute_change_set(
				ChangeSetName=changeStackName,
				StackName=stackName
			)

		# Wait for the results
		while True:
			# Get the stack
			stack = client.describe_stacks(StackName=stackName)

			# Try to get the status
			try:
				stackStatus = stack["Stacks"][0]["StackStatus"]
			except Exception:
				stackStatus = "CREATE_IN_PROGRESS"

			# Check the status
			if "IN_PROGRESS" in stackStatus:
				pass
			elif stackStatus[-7:] == "_FAILED" or "ROLLBACK" in stackStatus:
				return False
			elif stackStatus[-9:] == "_COMPLETE":
				return True

			# Wait a little
			time.sleep(15)

	def _checkParameters(self, parameters):
		""" Check if there are other parameters we need information about
		"""
		# Walk through the parameters
		for parameterName in parameters:
			# Check if there is an value
			if parameters[parameterName] in ["String", "Number", "List", "CommaDelimitedList"]:
				# Check if we have the parameter in de project config
				parametersConfig = ConfigurationTool.getConfig(ConfigKey.PARAMETERS)
				if parametersConfig is None:
					parametersConfig = {}

				# Check if the environment exists in the config
				if str(self.environment) in parametersConfig:
					if parameterName in parametersConfig[str(self.environment)]:
						parameters[parameterName] = parametersConfig[str(self.environment)][parameterName]
						continue

				# Ask for the value
				parameterValue, save = self._askForParameterValue(parameterName)

				# Save the value
				parameters[parameterName] = parameterValue

				# Check if we should save the value
				if save is True:
					# Check if the environment exists
					if str(self.environment) not in parametersConfig:
						parametersConfig[str(self.environment)] = {}

					# Set the value in de config
					parametersConfig[str(self.environment)][parameterName] = parameterValue

					# Save the value
					ConfigurationTool.setConfig(ConfigKey.PARAMETERS, parametersConfig, ConfigType.PROJECT)
					ConfigurationTool.saveConfig(ConfigType.PROJECT)

		return parameters

	def _askForParameterValue(self, parameterName):
		""" Ask for the value of an parameter

			Parameters:
				parameterName: Name of the parameter

			Returns tuple (parameterValue, save)
		"""
		# Ask the questions
		answer = prompt([{
			"type": "input",
			"name": "value",
			"message": "What is the value of parameter %s?" % parameterName
		}, {
			"type": "confirm",
			"message": "Do you want to save this value?",
			"name": "save"
		}])

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return (answer['value'], answer['save'])



import os
from setuptools import setup, find_packages
import tpd_pete


# Read the contents of the README file
folderPath = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(folderPath, "README.md"), encoding="utf-8") as f:
	longDescription = f.read()


# Generate setup config
setup(
	name="tpd_pete",
	version=tpd_pete.VERSION,
	description="TPD Pete is a AWS Deployment tool for AWS Cloudformation",
	author="TotalPunch Development",
	author_email="info@totalpunch.nl",
	url="https://github.com/totalpunch/TPD-Pete",
	download_url="https://github.com/totalpunch/TPD-Pete/archive/%s.tar.gz" % tpd_pete.VERSION,
	license="MIT",
	packages=find_packages(),
	entry_points={
		"console_scripts": [
			"pete = tpd_pete.__main__:main"
		]
	},
	install_requires=[
		"boto3==1.16.22",
		"pyinquirer==1.0.3",
		"halo==0.0.29",
		"termcolor==1.1.0",
		"cfn_flip==1.2.2"
	],
	zip_safe=False,
	python_requires=">=3.7",
	long_description=longDescription,
	long_description_content_type="text/markdown"
)



import os
import boto3


class BotoTool(object):
	@classmethod
	def getRegions(cls):
		""" Get all AWS regions
		"""
		# Get a boto client
		client = cls._getClient("ec2", region="us-east-1")

		# Retrieve all the regions
		regions = [region["RegionName"] for region in client.describe_regions()["Regions"]]
		return regions

	@classmethod
	def getRegion(cls, profile):
		""" Get AWS region of a profile
		"""
		# Check if there is an environment var
		if "AWS_DEFAULT_REGION" not in os.environ:
			raise Exception("Could not find a default region, please specify environment variable AWS_DEFAULT_REGION")

		return os.environ["AWS_DEFAULT_REGION"]

	@classmethod
	def getProfiles(cls):
		""" Get all AWS profiles
		"""
		return ["default"]

	@classmethod
	def getS3Buckets(cls, profile=None):
		""" Get your S3 bucket
		"""
		# Get the boto3 client
		client = cls._getClient("s3", region="us-east-1", profile=profile)

		# Retrieve the bucket names
		buckets = client.list_buckets()

		# Get the names
		bucketNames = [bucket["Name"] for bucket in buckets]

		return bucketNames

	@classmethod
	def uploadToS3(cls, fromPath, toBucket, toKey, region, profile=None):
		""" Upload a file to S3
		"""
		# Get a boto3 client
		client = cls._getClient("s3", region=region, profile=profile)

		# Upload the file
		client.upload_file(fromPath, toBucket, toKey)
		return "https://%s.s3.%s.amazonaws.com/%s" % (toBucket, region, toKey)

	@classmethod
	def _getClient(cls, resourceType, region, profile=None):
		""" Get a Boto3 client
		"""
		# Check if the profile name is set
		if profile is not None and profile != "default":
			# Remember if we should setup the profile
			shouldSetupProfile = True

			# Check if we are in CI mode
			if "PETE_CI" in os.environ:
				# Check if we already set the ACCESS_KEY
				if "AWS_ACCESS_KEY_ID" in os.environ:
					# Then skip the setup of the profile
					shouldSetupProfile = False

			# Check if we should setup the profile
			if shouldSetupProfile is True:
				# Add the profile name to environment variables
				os.environ["AWS_PROFILE"] = profile

		return boto3.client(resourceType, region_name=region)



import subprocess
from .boto import BotoTool


class AWSCliTool(object):
	@classmethod
	def getRegion(cls, profile):
		""" Get AWS region of a profile
		"""
		# Check if the AWS Cli is available
		if cls.hasAWSCli() is False:
			# Use boto3
			return BotoTool.getRegion(profile)

		# Open the AWS configuration
		command = "cat ~/.aws/config"

		# Request the config from AWS Cli
		result = subprocess.run(command, shell=True, capture_output=True)

		# Check the response
		if result.returncode != 0:
			raise Exception("Could not successfully get the profiles from AWS Cli. Has you run `aws configure`?")

		# Parse the output
		lines = (result.stdout).decode()

		# Remember the region and profile of the config
		region = None
		configProfile = None

		# Walk throught the lines
		for line in lines.split("\n"):
			# Check line length
			if len(line) == 0:
				continue

			# Check if the next lines belong to a profile
			if line[0] == "[":
				# Save the config profile
				configProfile = line[1:-1]

			# Check if this is the region
			elif line[:6] == "region":
				# Check the profile
				if configProfile == "default" and region is None:
					region = line[9:]
				elif configProfile == profile:
					region = line[9:]

		return region

	@classmethod
	def getProfiles(cls):
		""" Get all AWS profiles
		"""
		# Check if the AWS Cli is available
		if cls.hasAWSCli() is False:
			# Use boto3
			return BotoTool.getProfiles()

		# Build the command
		command = "cat ~/.aws/credentials | grep -o '\[[^]]*\]'"

		# Request all the profiles from AWS Cli
		result = subprocess.run(command, shell=True, capture_output=True)

		# Check the response
		if result.returncode != 0:
			raise Exception("Could not successfully get the profiles from AWS Cli. Has you run `aws configure`?")

		# Parse the output
		output = (result.stdout).decode()

		return [(line[1:-1]).strip() for line in output.split("\n")]

	@classmethod
	def getS3Buckets(cls, profile=None):
		""" Get your S3 bucket
		"""
		# Check if the AWS Cli is available
		if cls.hasAWSCli() is False:
			# Use boto3
			return BotoTool.getS3Buckets(profile)

		# Build the command
		command = "aws s3 ls"

		# Check if there is an profile
		if profile is not None:
			command = command + " --profile %s" % profile

		# Execute the command
		result = subprocess.run(command, shell=True, capture_output=True)

		# Check the response
		if result.returncode != 0:
			raise Exception("Could not successfully get the S3 buckets from AWS Cli. Do you have the right permissions?")

		# Parse the output
		output = (result.stdout).decode()

		return [(line[19:]).strip() for line in output.split("\n")]

	@classmethod
	def hasAWSCli(cls):
		""" Check if the AWS Cli is available
		"""
		# Build the command
		command = "aws help"

		# Request the config from AWS Cli
		result = subprocess.run(command, shell=True, capture_output=True)

		# Check the response
		if result.returncode != 0:
			return False
		return True
